Package: RBesT
Type: Package
Title: R Bayesian Evidence Synthesis Tools
Description: Tool-set to support Bayesian evidence synthesis.  This
    includes meta-analysis, (robust) prior derivation from historical
    data, operating characteristics and analysis (1 and 2 sample
    cases). Please refer to Weber et al. (2021) <doi:10.18637/jss.v100.i19>
    for details on applying this package while Neuenschwander et al. (2010)
    <doi:10.1177/1740774509356002> and Schmidli et al. (2014)
    <doi:10.1111/biom.12242> explain details on the methodology.
Version: 1.11-0
Authors@R: c(person("Novartis", "Pharma AG", role = "cph")
            ,person("Sebastian", "Weber", email="sebastian.weber@novartis.com", role=c("aut", "cre"))
            ,person("Beat", "Neuenschwander", email="beat.neuenschwander@novartis.com", role="ctb")
            ,person("Heinz", "Schmidli", email="heinz.schmidli@novartis.com", role="ctb")
            ,person("Baldur", "Magnusson", email="baldur.magnusson@novartis.com", role="ctb")
            ,person("Yue", "Li", email="yue-1.li@novartis.com", role="ctb")
            ,person("Satrajit", "Roychoudhury", email="satrajit.roychoudhury@novartis.com", role="ctb")
            ,person("Lukas A.", "Widmer", email="lukas_andreas.widmer@novartis.com", role = "ctb", comment = c(ORCID = "0000-0003-1471-3493"))
            ,person("Daniel", "Sabanés Bové", email="daniel.sabanes_bove@rconis.com", role = "ctb", comment = c(ORCID = "0000-0002-0176-9239"))
            ,person("Trustees of", "Columbia University", role="cph", comment="R/stanmodels.R, configure, configure.win")
	    )
Depends: R (>= 3.5.0)
Imports: methods, Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1), rstan (>=
        2.32.0), rstantools (>= 2.4.0), posterior (>= 1.5.0),
        assertthat, mvtnorm, Formula, checkmate, bayesplot (>= 1.4.0),
        ggplot2, dplyr, stats, utils, matrixStats, statmod, abind,
        rlang, jsonlite, lifecycle, Rdpack
LinkingTo: BH (>= 1.72.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.32.0), StanHeaders (>=
        2.32.0)
License: GPL (>= 3)
LazyData: true
Biarch: true
NeedsCompilation: yes
UseLTO: true
URL: https://opensource.nibr.com/RBesT/
BugReports: https://github.com/Novartis/RBesT/issues
Suggests: rmarkdown, knitr, MASS, testthat (>= 2.0.0), foreach, purrr,
        rstanarm (>= 2.17.2), scales, tools, broom, tidyr, parallel,
        brms, glue, ragg, withr, mirai, parallelly
VignetteBuilder: knitr
SystemRequirements: GNU make, pandoc (>= 1.12.3), pngquant, C++17
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RdMacros: Rdpack
Packaged: 2026-08-04 12:40:29 UTC; runner
Author: Novartis Pharma AG [cph],
  Sebastian Weber [aut, cre],
  Beat Neuenschwander [ctb],
  Heinz Schmidli [ctb],
  Baldur Magnusson [ctb],
  Yue Li [ctb],
  Satrajit Roychoudhury [ctb],
  Lukas A. Widmer [ctb] (ORCID: <https://orcid.org/0000-0003-1471-3493>),
  Daniel Sabanés Bové [ctb] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Trustees of Columbia University [cph] (R/stanmodels.R, configure,
    configure.win)
Maintainer: Sebastian Weber <sebastian.weber@novartis.com>
Repository: CRAN
Date/Publication: 2026-08-04 16:40:09 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-04 18:38:08 UTC; unix
Archs: RBesT.so.dSYM
